#!/bin/bash

################################
##/////////////////////)######## Onboarding with DEPNotify
##|                      )######
##|                        )#### Provides a user-facing onboarding walkthrough
##|                         )###
##|         #######)         )## TYPE: Jamf Policy Script
##|         ########)        )##
##|         #######)        )### Parameters:
##|                         )### $1-3 - Reserved by Jamf
##|                       )##### $4 - Options for DEPNotify (see code)
##|                      )###### $5 - Main Title Starting Text (optional)
##|      |  ####\         \##### $6 - Path to icon (optional)
##|      |  #####\         \#### $7 - Window Title (not normally visible, optional)
##|    | |  ######\         \###
##|  | | |  #######\         \##
################################
##
## Latest version and additional notes available at our GitHub
## 	https://github.com/Rocketman-Tech/Onboarding-With-DEPNotify
##
################################################################################

##            ##
## Parameters ##
##            ##
DNOPTS=$4 ## Options to pass to DEPNotify
## Most common options:
##
## -fullScreen
## This flag will create a full screen behind the DEPNotify screen to focus the
## user on the task at hand. By default, DEPNotify launches as a window that
## can be moved by the end user. Additionally, command-control-x will quit
## DEPNotify, although this can be modified via the DEPNotify configuration.
##
## -jamf
## This has DEP Notify read in the Jamf log at /var/log/jamf.log and then
## update the status line in the DEP Notify window with any installations or
## policy executions from the Jamf log. Note there is nothing special you need
## to name your items in Jamf for them to be read.
##
## See more at: https://gitlab.com/Mactroll/DEPNotify#application-flags
MAINTITLE=$5 ## Initial title - defaults to "Welcome to your new Mac!"
LOGOFILE=$6 ## Full path to logo - must be previously deployed to work
WINDOWTITLE=$7 ## Title of window - not normally visiable to users

function coffee {

	## Disable sleep for duration of run
	/usr/bin/caffeinate -d -i -m -u &
	caffeinatepid=$!
}

function pauseJamfFramework {

	## Update Jamf frameworks
	/usr/local/bin/jamf manage

	## Disable Jamf Check-Ins
	jamftasks=($( find /Library/LaunchDaemons -iname "*task*" -type f -maxdepth 1 ))
	for ((i=0;i<${#jamftasks[@]};i++))
	do
		/bin/launchctl unload -w "${jamftasks[$i]}"
	done

	## Kill any check-in in progress
	jamfpid=$( ps -ax | grep "jamf policy -randomDelaySeconds" | grep -v "grep" | awk '{ print $1 }' )
	if [ "$jamfpid" != "" ];
	then
		kill -9 "$jamfpid"
	fi
}

function waitForUser {

	## Check to see if we're in a user context or not. Wait if not.
	dockStatus=$( /usr/bin/pgrep -x Dock )
	while [[ "$dockStatus" == "" ]]; do
		sleep 1
		dockStatus=$( /usr/bin/pgrep -x Dock )
	done

	## Get the current user?
	currentuser=$( /usr/sbin/scutil <<< "show State:/Users/ConsoleUser" | /usr/bin/awk -F': ' '/[[:space:]]+Name[[:space:]]:/ { if ( $2 != "loginwindow" ) { print $2 }}' )
	userid=$( /usr/bin/id -u $currentuser )
}

function startDEPNotify {

	## Create the depnotify log file
	/usr/bin/touch /var/tmp/depnotify.log
	/bin/chmod 777 /var/tmp/depnotify.log

	## Set up the initial DEP Notify window
	/bin/echo "Command: Image: ${LOGOFILE}" >> /var/tmp/depnotify.log
	/bin/echo "Command: WindowTitle: ${WINDOWTITLE}" >> /var/tmp/depnotify.log
	/bin/echo "Command: MainTitle: ${MAINTITLE}" >> /var/tmp/depnotify.log

	## Load DEP Notify
	deploc=$( /usr/bin/find /Applications -maxdepth 2 -type d -iname "*DEP*.app" )
	/bin/launchctl asuser $userid "$deploc/Contents/MacOS/DEPNotify" ${DNOPTS} 2>/dev/null &
	deppid=$!
}

function cleanUp {

	## Re-enable Jamf management
	for ((i=0;i<${#jamftasks[@]};i++))
	do
		/bin/launchctl load -w "${jamftasks[$i]}"
	done

	## Quit DEPNotify
	/bin/echo "Command: Quit" >> /var/tmp/depnotify.log
	/bin/rm -rf "$deploc" ## Deletes the DEPNotify.app

	## Delete temp files
	/bin/rm /var/tmp/depnotify.log
	/usr/bin/defaults delete menu.nomad.DEPNotify

	## Disable Caffeine
	/bin/kill "$caffeinatepid"
}

function DEPNotify {

	local NotifyCommand=$1
	/bin/echo "$NotifyCommand" >> /var/tmp/depnotify.log
}

function jamfCommand {

	local jamfTrigger=$1

	if [[ $jamfTrigger == "recon" ]]; then
		/usr/local/bin/jamf recon
	elif [[ $jamfTrigger == "policy" ]]; then
		/usr/local/bin/jamf policy
	else
		/usr/local/bin/jamf policy -event $jamfTrigger
	fi
}

###							###
### Main Script	###
###							###

## These next four lines execute functions above
coffee				## Uses 'caffeinate' to disable sleep and stores the PID for later
pauseJamfFramework 		## Disables recurring Jamf check-ins to prevent overlaps
waitForUser 			## Blocking loop; Waits until DEP is complete and user is logged in
startDEPNotify 			## Initial setup and execution of DEPNotify as user

###                      ###
### YOUR STUFF GOES HERE ###
###                      ###

## NOTES:
##	There are two functions to help simplify your DEPNotify commands and
##		calls to Jamf for other policies.
##
##	1. DEPNotify - Appends text to /var/tmp/depnotify.log
##		Ex. DEPNotify "Command: MainText: Message goes here"
##			DEPNotify "Status: Tell the user what we are doing..."
##
##	2. jamfCommand - Simplifies calls to the jamf binary with three options
##		'recon' 	- Submits an inventory to udpate Smart Groups, etc.
##		'policy' 	- Makes a normal policy check for new applicable policies
##		other 		- Calls jamf policy with the passed in argument as a manual trigger
##			Ex. "jamfCommand renameComputer" - executes "/usr/local/bin/jamf policy -trigger renameComputer"
##
## EXAMPLE WORKFLOW
##
# ## Machine Configuration
# DEPNotify "Command: MainText: Configuring Machine."
# DEPNotify "Status: Setting Computer Name"
# jamfCommand configureComputer
#
# ## Installers required for every Mac - Runs policies with 'deploy' manual trigger
# DEPNotify "Status: Starting Deployment"
# DEPNotify "Command: MainText: Starting software deployment.\n\nThis process can take some time to complete."
# jamfCommand deploy
# sleep 3
#
# ## Add Departmental Apps - Run polices with "installDepartmentalApps" manual and scoped to departments
# DEPNotify "Command: MainText: Adding Departmental Components."
# DEPNotify "Status: Adding Departmental Applications. This WILL take a while."
# jamfCommand recon
# sleep 1
# jamfCommand installDepartmentalApps
#
# ## Send updated inventory for Smart Groups and check for any remaining scoped policies
# DEPNotify "Command: MainText: Final install checks."
# DEPNotify "Status: Update inventory record."
# jamfCommand recon
# sleep 1
# DEPNotify "Status: Final policy check."
# jamfCommand policy

###         ###
### Cleanup ###
###         ###
cleanUp ## Quits application, deletes temporary files, and resumes normal operation
